/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Romain Rouvoy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: FcExplorerImpl.java,v 1.3 2004/11/15 16:44:25 moroy Exp $
 ====================================================================*/

package org.objectweb.fractal.explorer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.event.KeyEvent;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.parser.api.ParserConfiguration;
import org.objectweb.util.explorer.swing.api.Explorer;
import org.objectweb.util.explorer.swing.api.StatusBar;
import org.objectweb.util.explorer.swing.api.ViewPanel;
import org.objectweb.util.explorer.swing.lib.DefaultTreePanel;
import org.objectweb.util.trace.TraceSystem;

/**
 * This Fractal component contains the explorer
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class FcExplorerImpl
  implements BindingController, 
             FcExplorerAttributes 
{
    
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================
    
    /** The string which represents the fcAppl client */
    protected final String FC_APPL = "fcAppl";
    
    /** The explorer component. */
    protected Component explorer_ = null;
    
    /** The tree interface to use. */
    protected Tree tree_ = null;
    
    /** The explorer interface to use. */
    protected Explorer explorerItf_ = null;
    
    /** The config files */
    protected String configFiles_ = null;
    
    /** The  list of Component */
    protected Map elements_ = new HashMap();
    
    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================
    
    /** Returns an ImageIcon, or null if the path is invalid. */
    protected static ImageIcon createIcon(String imageName) {
        if (imageName != null) {
            URL urlFile = null;
            urlFile = Thread.currentThread().getContextClassLoader().getResource(imageName);            
            if(urlFile==null){            
                try {
                    urlFile = new URL(imageName);
                } catch (java.net.MalformedURLException e) {
                    System.out.println(imageName + ": Malformed URL !");
                }
            }
            if(urlFile!=null){
                return new ImageIcon(urlFile);
            }
        }
        return null;
    }
    
    protected void createFileMenu(JMenuBar menu, JToolBar toolBar) {
        JMenu fileMenu = new JMenu("Explorer");
        fileMenu.setMnemonic(KeyEvent.VK_E);
        
        JMenuItem menuItem = null;
        JButton button = null;
        
        Action refreshAction = new RefreshAction("Refresh the tree", createIcon("icons/Reload.png"), "Refreshes the tree", new Integer(KeyEvent.VK_R));
        Action quitAction = new QuitAction("Quit", createIcon("icons/Power.png"), "Quit the application", new Integer(KeyEvent.VK_Q));
        
        // Add to the menubar
        
        menuItem = new JMenuItem(refreshAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, Event.CTRL_MASK));
        fileMenu.add(menuItem);
        
        menuItem = new JMenuItem(quitAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, Event.ALT_MASK));
        fileMenu.add(menuItem);
        
        // Adding to the toolbar
        
        button = new JButton(quitAction);
        button.setText("");
        toolBar.add(button);
        
        button = new JButton(refreshAction);
        button.setText("");
        toolBar.add(button);
        
        menu.add(fileMenu);
    }
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    /**
     * Default constructor
     */
    public FcExplorerImpl(){
        TraceSystem.setLevel("debug");
        try {
            // Creates a Fractal Explorer instance.
            Factory f = FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND);
            explorer_ = (Component)f.newComponent("org.objectweb.fractal.explorer.SwingFractalExplorer",null);
            Fractal.getLifeCycleController(explorer_).startFc();

            // Browses the structure of the Explorer component.
            Tree treeItf = (Tree)explorer_.getFcInterface(Tree.TREE);
            
            // Loads the Explorer Framework configuration file from the fractal-explorer Java archive.
            setConfigFiles("config/fractalProperties.xml");
        
            // Populates the tree.
            tree_ = (Tree)explorer_.getFcInterface(Tree.TREE);

            JMenuBar menuBar = new JMenuBar();
            JToolBar toolBar = new JToolBar();        
            createFileMenu(menuBar, toolBar);
            
            explorerItf_ = (Explorer)explorer_.getFcInterface(Explorer.EXPLORER);
            explorerItf_.setMenuBar(menuBar);
            explorerItf_.setToolBar(toolBar);
            
            // Configures the main frame.
            explorerItf_ = (Explorer)explorer_.getFcInterface(Explorer.EXPLORER);
            ViewPanel viewPanelItf = (ViewPanel)explorer_.getFcInterface(ViewPanel.VIEW_PANEL);
            StatusBar statusBar = (StatusBar)explorer_.getFcInterface(StatusBar.STATUS_BAR);
            JFrame frame = new JFrame("Fractal Explorer");
            frame.setJMenuBar(menuBar);
            frame.getContentPane().setLayout(new BorderLayout());
            frame.getContentPane().add(toolBar,BorderLayout.NORTH);
            frame.getContentPane().add(new JSplitPane(
                    JSplitPane.HORIZONTAL_SPLIT,
                    true,
                    new DefaultTreePanel(explorerItf_.getTree()),
                    viewPanelItf.getViewPanel()), BorderLayout.CENTER);
            frame.getContentPane().add(statusBar.getStatusBar(), BorderLayout.SOUTH);
            frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            frame.pack();
            
            // Centers the frame
            Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
            frame.setLocation((screenSize.width - frame.getWidth()) / 2, (screenSize.height - frame.getHeight()) / 2);
            
            frame.setVisible(true);
            
        } catch (ADLException e) {
            e.printStackTrace();
        } catch (IllegalLifeCycleException e) {
            e.printStackTrace();
        } catch (NoSuchInterfaceException e) {
            e.printStackTrace();
        }        
    }
    
    // ==================================================================
    //
    // Public methods for interface ExplorerAttributes.
    //
    // ==================================================================
    
    public void setConfigFiles (String configFiles) {
        if(configFiles != null){
            try {
                configFiles_ = configFiles;
                ParserConfiguration parser = (ParserConfiguration)explorer_.getFcInterface(ParserConfiguration.PARSER_CONFIGURATION);
                StringTokenizer st = new StringTokenizer(configFiles,"; ");
                List l = new Vector();
                while (st.hasMoreTokens()) { 
                    String theFile = st.nextToken();
                    parser.addPropertyFile(theFile);
                }
                parser.parse();
                
                Explorer explorerItf = (Explorer)explorer_.getFcInterface(Explorer.EXPLORER);
                explorerItf.setCurrentRoles(new String[]{"administrator"});
            } catch (NoSuchInterfaceException e) {
                e.printStackTrace();
            }
        }
    }
    
    public String getConfigFiles () {
        return configFiles_;
    }
    
    // ==================================================================
    //
    // Public methods for interface BindingController.
    //
    // ==================================================================
    
    public void bindFc(String clientItfName, Object sItf)
    throws
    NoSuchInterfaceException,
    IllegalBindingException,
    IllegalLifeCycleException {
        if (clientItfName.startsWith(FC_APPL)) {
            elements_.put(clientItfName, sItf);
            Tree treeItf = (Tree)explorer_.getFcInterface(Tree.TREE);
            treeItf.addEntry(FcExplorer.getName((Component)sItf), sItf,1);
        }
    }
    
    public String[] listFc() {
        return (String[])elements_.keySet().toArray(new String[elements_.size()]);
    }
    
    public Object lookupFc(String clientItfName)
    throws NoSuchInterfaceException {
        if (clientItfName.startsWith(FC_APPL)) {
            return elements_.get(clientItfName);
        }
        return null;
    }
    
    public void unbindFc(String clientItfName)
    throws
    NoSuchInterfaceException,
    IllegalBindingException,
    IllegalLifeCycleException {
        if (clientItfName.equals(FC_APPL)) {
            elements_.remove(clientItfName);   
        }
    }
    
    // ==================================================================
    //
    // Internal classes.
    //
    // ==================================================================
    
    protected class RefreshAction extends AbstractAction {
        
        public RefreshAction(String nom, ImageIcon image, String desc, Integer mnemonic) {
            super(nom, image);
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }
        
        public void actionPerformed(java.awt.event.ActionEvent ae) {
            FcExplorerImpl.this.tree_.refreshAll();
        }
    }
    
    protected class QuitAction extends AbstractAction {
        
        public QuitAction(String nom, ImageIcon image, String desc, Integer mnemonic) {
            super(nom, image);
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }
        
        public void actionPerformed(java.awt.event.ActionEvent ae) {
            System.exit(0);
        }
    }
    
}