/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Jerome Moroy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: ComponentContextForService.java,v 1.3 2004/11/15 16:44:24 moroy Exp $
 ====================================================================*/

package org.objectweb.fractal.explorer.context;

import java.util.Enumeration;
import java.util.Hashtable;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.explorer.FcExplorer;
import org.objectweb.util.explorer.api.Context;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.common.api.ContextContainer;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;

/**
 * Component context for user role. 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class ComponentContextForService 
  implements Context 
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /** This Context Container contains the controller interfaces. */
    protected ContextContainer controllers_ = null;
    
    // ==================================================================
    //
    // No Constructors.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Return true if the given Interface is a Client interface.
     */
    protected boolean isClient(Interface ir) {
        InterfaceType it = (InterfaceType) ir.getFcItfType();
        return it.isFcClientItf();
    }
    
    /**
     * Return true if the given Interface is a Collection Client interface.
     */
    protected boolean isClientCollection(Interface ir) {
        InterfaceType it = (InterfaceType)ir.getFcItfType();
        return (it.isFcClientItf() && it.isFcCollectionItf());
    }
    
    /**
     * Return true if the given Interface is a Control interface.
     */
    protected boolean isController(Interface itf) {
        return (itf.getFcItfName().endsWith("-controller")||itf.getFcItfName().endsWith("factory"));
    }
    
    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================
    
    /**
     * Returns an array containing the entries contained by the target context.
     * @return A new array of entry.
     */
    public Entry[] getEntries(Object object) {
        Component ci = (Component)object;
        
        Hashtable collectionInterfaces = new Hashtable();
        java.util.List values = new java.util.ArrayList();
        
        controllers_ = new ControllerContainer();
        Object[] fcItfs = (Object[])ci.getFcInterfaces();
        
        for (int i=0 ; i<fcItfs.length ; i++) {
            if (!ci.equals(fcItfs[i])) {
                Interface itf = (Interface)fcItfs[i];
                if (isController(itf)) {
                    controllers_.addEntry(FcExplorer.getName(itf),itf);
                } else if (isClientCollection(itf)) {                    
                    InterfaceType it = (InterfaceType)itf.getFcItfType();
                    String name = FcExplorer.getName(itf);
                    if(!collectionInterfaces.containsKey(it.getFcItfName())){
                        ContextContainer cc = new ClientCollectionInterfaceContainer(itf);
                        cc.addEntry(name,new ClientCollectionInterfaceWrapper(itf));
                        collectionInterfaces.put(it.getFcItfName(),cc);
                    } else {
                        ((ContextContainer)collectionInterfaces.get(it.getFcItfName())).addEntry(name,
                                                                                                 new ClientInterfaceWrapper(itf));
                    }
                } else if(isClient(itf)) {
                    values.add(new DefaultEntry(FcExplorer.getName(itf), new ClientInterfaceWrapper(itf)));
                } else {
                    values.add(new DefaultEntry(FcExplorer.getName(itf), itf));
                }
            }
        }
        // Add the collection interfaces
        Enumeration keys,elements;
        keys = collectionInterfaces.keys();
        elements = collectionInterfaces.elements();
        for (int i = 0 ; i < collectionInterfaces.size() ; i++) {
            values.add(new DefaultEntry((String) keys.nextElement(),elements.nextElement())); 
        }
        
        return (Entry[])values.toArray(new Entry[0]);        
    }
        
}






